; gpio.s
; Desenvolvido para a placa EK-TM4C1294XL
; Uso de GPIO com interrupo ...
; Prof. Guilherme de S. Peron	- 12/03/2018
; Prof. Marcos E. P. Monteiro	- 12/03/2018
; Prof. DaLuz           		- 25/02/2022

;################################################################################
        THUMB                        ; Instrues do tipo Thumb-2
;################################################################################
; Definies de Valores
BIT0						EQU		2_0001
BIT1						EQU		2_0010
; Definies dos Registradores Gerais
SYSCTL_RCGCGPIO_R	 		EQU		0x400FE608
SYSCTL_PRGPIO_R		 		EQU		0x400FEA08
; NVIC
NVIC_EN1_R					EQU		0xE000E104
NVIC_PRI12_R		     	EQU		0xE000E430        
; Definies dos Ports - PORT J
GPIO_PORTJ_AHB_LOCK_R    	EQU    0x40060520
GPIO_PORTJ_AHB_CR_R      	EQU    0x40060524
GPIO_PORTJ_AHB_AMSEL_R   	EQU    0x40060528
GPIO_PORTJ_AHB_PCTL_R    	EQU    0x4006052C
GPIO_PORTJ_AHB_DIR_R     	EQU    0x40060400
GPIO_PORTJ_AHB_AFSEL_R   	EQU    0x40060420
GPIO_PORTJ_AHB_DEN_R     	EQU    0x4006051C
GPIO_PORTJ_AHB_PUR_R     	EQU    0x40060510	
GPIO_PORTJ_AHB_DATA_R    	EQU    0x400603FC
GPIO_PORTJ_AHB_DATA_BITS_R  EQU    0x40060000
GPIO_PORTJ               	EQU    2_000000100000000
; PORT N
GPIO_PORTN_AHB_LOCK_R    	EQU    0x40064520
GPIO_PORTN_AHB_CR_R      	EQU    0x40064524
GPIO_PORTN_AHB_AMSEL_R   	EQU    0x40064528
GPIO_PORTN_AHB_PCTL_R    	EQU    0x4006452C
GPIO_PORTN_AHB_DIR_R     	EQU    0x40064400
GPIO_PORTN_AHB_AFSEL_R   	EQU    0x40064420
GPIO_PORTN_AHB_DEN_R     	EQU    0x4006451C
GPIO_PORTN_AHB_PUR_R     	EQU    0x40064510	
GPIO_PORTN_AHB_DATA_R    	EQU    0x400643FC
GPIO_PORTN_AHB_DATA_BITS_R  EQU    0x40064000
GPIO_PORTN               	EQU    2_001000000000000	

;################################################################################
; rea de Cdigo - Tudo abaixo da diretiva a seguir ser armazenado na memria de 
;                  cdigo
        AREA    |.text|, CODE, READONLY, ALIGN=2

		; Se alguma funo do arquivo for chamada em outro arquivo	
        EXPORT GPIO_Init            			;Permite chamar GPIO_Init de outro arquivo
		EXPORT PortN_Output						;Permite chamar PortN_Output de outro arquivo
		EXPORT PortJ_Input          			;Permite chamar PortJ_Input de outro arquivo
		EXPORT PortN_Invertepino0   			;Permite chamar PortN_Invertepino0 de outro arquivo
									
;################################################################################
; Funo GPIO_Init
; Parmetro de entrada: No tem
; Parmetro de sada: No tem
;################################################################################
GPIO_Init
; 1. Ativar o clock para a porta setando o bit correspondente no registrador RCGCGPIO,
; aps isso verificar no PRGPIO se a porta est pronta para uso.
; enable clock to GPIOF at clock gating register
	LDR     R0, =SYSCTL_RCGCGPIO_R  			;Carrega o endereo do registrador RCGCGPIO
	MOV		R1, #GPIO_PORTN                 	;Seta o bit da porta N
	ORR     R1, #GPIO_PORTJ						;Seta o bit da porta J, fazendo com OR
	STR     R1, [R0]							;Move para a memria os bits das portas no endereo do RCGCGPIO
 
	LDR     R0, =SYSCTL_PRGPIO_R				;Carrega o endereo do PRGPIO para esperar os GPIO ficarem prontos
EsperaGPIO
	LDR     R1, [R0]							;L da memria o contedo do endereo do registrador
	MOV     R2, #GPIO_PORTN                 	;Seta os bits correspondentes s portas para fazer a comparao
	ORR     R2, #GPIO_PORTJ                 	;Seta o bit da porta J, fazendo com OR
    TST     R1, R2								;Testa o R1 com R2 fazendo R1 & R2
    BEQ     EsperaGPIO					    	;Se o flag Z=1, volta para o lao. Seno continua executando
; 2. Limpar o AMSEL para desabilitar a analgica
	MOV     R1, #0x00							;Colocar 0 no registrador para desabilitar a funo analgica
	LDR     R0, =GPIO_PORTJ_AHB_AMSEL_R     	;Carrega o R0 com o endereo do AMSEL para a porta J
	STR     R1, [R0]							;Guarda no registrador AMSEL da porta J da memria
	LDR     R0, =GPIO_PORTN_AHB_AMSEL_R			;Carrega o R0 com o endereo do AMSEL para a porta N
	STR     R1, [R0]					    	;Guarda no registrador AMSEL da porta N da memria
; 3. Limpar PCTL para selecionar o GPIO
	MOV     R1, #0x00					    	;Colocar 0 no registrador para selecionar o modo GPIO
	LDR     R0, =GPIO_PORTJ_AHB_PCTL_R			;Carrega o R0 com o endereo do PCTL para a porta J
	STR     R1, [R0]                        	;Guarda no registrador PCTL da porta J da memria
	LDR     R0, =GPIO_PORTN_AHB_PCTL_R      	;Carrega o R0 com o endereo do PCTL para a porta N
	STR     R1, [R0]                        	;Guarda no registrador PCTL da porta N da memria
; 4. DIR para 0 se for entrada, 1 se for sada
	LDR     R0, =GPIO_PORTN_AHB_DIR_R			;Carrega o R0 com o endereo do DIR para a porta N
	MOV     R1, #BIT0							;PN1 & PN0 para LED
	ORR     R1, #BIT1      						;Enviar o valor 0x03 para habilitar os pinos como sada
	STR     R1, [R0]							;Guarda no registrador
;O certo era verificar os outros bits da PJ para no transformar entradas em sadas desnecessrias
	LDR     R0, =GPIO_PORTJ_AHB_DIR_R			;Carrega o R0 com o endereo do DIR para a porta J
	MOV     R1, #0x00               			;Colocar 0 no registrador DIR para funcionar com sada
	STR     R1, [R0]							;Guarda no registrador PCTL da porta J da memria
; 5. Limpar os bits AFSEL para 0 para selecionar GPIO 
;    Sem funo alternativa
	MOV     R1, #0x00							;Colocar o valor 0 para no setar funo alternativa
	LDR     R0, =GPIO_PORTN_AHB_AFSEL_R			;Carrega o endereo do AFSEL da porta N
	STR     R1, [R0]							;Escreve na porta
	LDR     R0, =GPIO_PORTJ_AHB_AFSEL_R     	;Carrega o endereo do AFSEL da porta J
	STR     R1, [R0]                        	;Escreve na porta
; 6. Setar os bits de DEN para habilitar I/O digital
	LDR     R0, =GPIO_PORTN_AHB_DEN_R			;Carrega o endereo do DEN
	LDR     R1, [R0]							;Ler da memria o registrador GPIO_PORTN_AHB_DEN_R
	MOV     R2, #BIT0
	ORR     R2, #BIT1							;Habilitar funcionalidade digital na DEN os bits 0 e 1
	ORR     R1, R2
	STR     R1, [R0]							;Escreve no registrador da memria funcionalidade digital 
	LDR     R0, =GPIO_PORTJ_AHB_DEN_R			;Carrega o endereo do DEN
	LDR     R1, [R0]                        	;Ler da memria o registrador GPIO_PORTN_AHB_DEN_R
	MOV     R2, #BIT0                           
	ORR     R2, #BIT1			            	;Habilitar funcionalidade digital na DEN os bits 0 e 1
	ORR     R1, R2                              
	STR     R1, [R0]                        	;Escreve no registrador da memria funcionalidade digital
; 7. Para habilitar resistor de pull-up interno, setar PUR para 1
	LDR     R0, =GPIO_PORTJ_AHB_PUR_R			;Carrega o endereo do PUR para a porta J
	MOV     R1, #BIT0							;Habilitar funcionalidade digital de resistor de pull-up 
	ORR     R1, #BIT1							;nos bits 0 e 1
	STR     R1, [R0]							;Escreve no registrador da memria do resistor de pull-up
	BX      LR

;################################################################################
; Funo PortN_Output
; Parmetro de entrada: R0 --> se os BIT1 e BIT0 esto ligado ou desligado
; Parmetro de sada: No tem
;################################################################################
PortN_Output
	LDR		R1, =GPIO_PORTN_AHB_DATA_BITS_R		;Carrega o valor do offset do data register
	ADD 	R1, #0x0C							;Soma ao offset o endereo do bit 1 para ser 
												;uma escrita amigvel
	STR 	R0, [R1]                        	;Escreve no barramento de dados na porta N1 somente
	BX 	LR										;Retorno
	
;################################################################################
; Funo PortN_Inverte0
; Parmetro de entrada: R1 --> se o BIT1 est ligado ou desligado
; Parmetro de sada: No tem
;################################################################################
PortN_Invertepino0
	PUSH 	{LR}
	LDR		R1, =GPIO_PORTN_AHB_DATA_BITS_R		;Carrega o valor do offset do data register
	ADD 	R1, #0x04							;Soma ao offset o endereo do bit 1 para ser 
												;uma leitura e escrita amigvel
	LDR 	R0, [R1]							;Carrega o valor atual para inverter
	EOR 	R0, R0, #1                      	;Inverte o bit 0
	STR 	R0, [R1]                        	;Escreve no barramento de dados na porta N0 somente
	POP 	{PC}								;Retorno	

;################################################################################
; Funo PortJ_Input
; Parmetro de entrada: No tem
; Parmetro de sada: R0 --> o valor da leitura
;################################################################################
PortJ_Input
	LDR		R1, =GPIO_PORTJ_AHB_DATA_BITS_R		;Carrega o valor do offset do data register
	ADD 	R1, #0x0C							;Soma ao offset o endereo dos bit 0 e 1 para 
												;serem os nicos a serem lidos tem uma leitura amigvel
	LDR 	R0, [R1]                            ;L no barramento de dados nos pinos J0 e J1 somente
	BX 		LR									;Retorno

    ALIGN                           			; garante que o fim da seo est alinhada 
    END                             			; fim do arquivo